package com.jianshui.invoice.service.impl;

import java.util.List;
import com.jianshui.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.jianshui.invoice.mapper.InvoiceFileMapper;
import com.jianshui.invoice.domain.InvoiceFile;
import com.jianshui.invoice.service.IInvoiceFileService;

/**
 * 发票URL存储Service业务层处理
 * 
 * @author kk
 * @date 2023-12-04
 */
@Service
public class InvoiceFileServiceImpl implements IInvoiceFileService 
{
    @Autowired
    private InvoiceFileMapper invoiceFileMapper;

    /**
     * 查询发票URL存储
     * 
     * @param id 发票URL存储主键
     * @return 发票URL存储
     */
    @Override
    public InvoiceFile selectInvoiceFileById(String id)
    {
        return invoiceFileMapper.selectInvoiceFileById(id);
    }

    /**
     * 查询发票URL存储列表
     * 
     * @param invoiceFile 发票URL存储
     * @return 发票URL存储
     */
    @Override
    public List<InvoiceFile> selectInvoiceFileList(InvoiceFile invoiceFile)
    {
        return invoiceFileMapper.selectInvoiceFileList(invoiceFile);
    }

    /**
     * 新增发票URL存储
     * 
     * @param invoiceFile 发票URL存储
     * @return 结果
     */
    @Override
    public int insertInvoiceFile(InvoiceFile invoiceFile)
    {
        invoiceFile.setCreateTime(DateUtils.getNowDate());
        return invoiceFileMapper.insertInvoiceFile(invoiceFile);
    }

    /**
     * 修改发票URL存储
     * 
     * @param invoiceFile 发票URL存储
     * @return 结果
     */
    @Override
    public int updateInvoiceFile(InvoiceFile invoiceFile)
    {
        return invoiceFileMapper.updateInvoiceFile(invoiceFile);
    }

    /**
     * 批量删除发票URL存储
     * 
     * @param ids 需要删除的发票URL存储主键
     * @return 结果
     */
    @Override
    public int deleteInvoiceFileByIds(String[] ids)
    {
        return invoiceFileMapper.deleteInvoiceFileByIds(ids);
    }

    /**
     * 删除发票URL存储信息
     * 
     * @param id 发票URL存储主键
     * @return 结果
     */
    @Override
    public int deleteInvoiceFileById(String id)
    {
        return invoiceFileMapper.deleteInvoiceFileById(id);
    }
}
