package com.jianshui.invoice.utils;

import com.alibaba.fastjson.JSONObject;
import com.jianshui.common.core.domain.AjaxResult;
import com.jianshui.invoice.domain.BillDetail;

import java.math.BigDecimal;

/**
 * @Description 用来将含税金额、不含税金额、税额等进行计算
 * @Author 巩权林
 * @Date 2021-09-29 19:49
 **/
public class BillDetailFormatUtil {

    /**
     * 计算明细的税额、含税金额、不含税金额
     *
     * @param billDetail
     * @return
     */
    public static BillDetail format(BillDetail billDetail) {
        Integer fphxz = billDetail.getFphxz();
        // 如果是被折扣行，直接返回
        if (fphxz != null && fphxz == 1) {
            return billDetail;
        }
        //如果单价或数量为空，直接返回
        if ((billDetail.getPrice().equals("") || billDetail.getPrice() == null) || (billDetail.getNum().equals("") || billDetail.getNum() == null)) {
            return billDetail;
        }
        // 如果存在扣除额，直接返回
        BigDecimal kce = billDetail.getKce();
        // 如果当前行存在扣除额
        if (kce != null && kce.compareTo(new BigDecimal(0)) != 0) {
            return billDetail;
        }

        Integer hsbz = billDetail.getHsbz();
        BigDecimal taxPrice = null; // 含税单价
        BigDecimal taxfreePrice = null; // 不含税单价
        BigDecimal tax = null; // 税额
        BigDecimal taxRate = billDetail.getTaxrate(); // 税率
        BigDecimal num = billDetail.getNum(); // 商品数量
        BigDecimal taxAmt = null; // 含税价格
        BigDecimal taxFreeAmt = null; // 不含税价格

        if (hsbz == 1) {
            taxPrice = billDetail.getPrice(); // 含税单价
            taxfreePrice = taxPrice.divide((new BigDecimal("1")).add(taxRate), 8, BigDecimal.ROUND_DOWN); // 计算不含税单价
            // 诺诺的文档公式如下，但是实际上开出来的票额和这个不一致： 在确认含税单价或者含税金额的情况下，含税标志为 1 含税，税额=round((数量*含税单价)*税率/(1+税率)),2)，不含税金额=含税金额-税额。
            // 需要先将含税单价转换为不含税单价，不含税单价进行计算，得到的结果才与票面一直
            // taxAmt = taxPrice.multiply(num);
            // tax = ((taxAmt.multiply(taxRate)).divide((new BigDecimal("1")).multiply(taxRate))).setScale(2, BigDecimal.ROUND_HALF_UP);
            // taxFreeAmt = taxPrice.subtract(tax);
        } else {
            taxfreePrice = (billDetail.getPrice()).setScale(8, BigDecimal.ROUND_DOWN); // 不含税单价
            // taxPrice = taxfreePrice.multiply((new BigDecimal("1")).add(taxRate)).setScale(9, BigDecimal.ROUND_HALF_UP); // 计算含税单价
            // 在确认不含税单价或者不含税金额的情况下，含税标志为 0 不含税，税额=round((数量*不含 税单价)*税率),2)，含税金额=不含税金额+税额。
            // tax = (taxfreePrice.multiply(num).multiply(taxRate)).setScale(2, BigDecimal.ROUND_HALF_UP); // 计算税额
            // taxAmt = taxFreeAmt.add(tax); // 计算含税金额
        }
        // 同一更新为不含税单价
        // TODO 如果税额和不含税金额使用ROUND_DOWN，最红结果会比用户结果小；如果为ROUND_HALF_UP，结果会比用户结果大；山钢项目要求开票税额要比他们的数据小，这里应该用ROUND_DOWN
        taxFreeAmt = (taxfreePrice.multiply(num)).setScale(8, BigDecimal.ROUND_HALF_UP);
        tax = (taxFreeAmt.multiply(taxRate)).setScale(2, BigDecimal.ROUND_HALF_UP); // 计算税额
        taxFreeAmt = taxFreeAmt.setScale(2, BigDecimal.ROUND_HALF_UP);
        taxAmt = taxFreeAmt.add(tax); // 计算含税金额
        billDetail.setHsbz(0);
        billDetail.setPrice(taxfreePrice);
        billDetail.setTaxamt(taxAmt);
        billDetail.setTaxfreeamt(taxFreeAmt);
        billDetail.setTax(tax);
        return billDetail;
    }

    /**
     * 计算明细的税额、含税金额、不含税金额
     *
     * @param billDetailRaw
     * @return
     */
    public static BillDetail formats(BillDetail billDetailRaw) {
        BillDetail billDetail = JSONObject.parseObject(JSONObject.toJSONString(billDetailRaw), BillDetail.class);
        Integer fphxz = billDetail.getFphxz();
        // 如果是被折扣行，直接返回
        if (fphxz == 1) {
            return billDetail;
        }
        //如果单价或数量为空，直接返回
        if (billDetail.getPrice() == null || billDetail.getNum() == null) {
            return billDetail;
        }
        // 如果存在扣除额，直接返回
        BigDecimal kce = billDetail.getKce();
        // 如果当前行存在扣除额
        if (kce != null && kce.compareTo(new BigDecimal(0)) != 0) {
            return billDetail;
        }

        Integer hsbz = billDetail.getHsbz();
        BigDecimal taxPrice = null; // 含税单价
        BigDecimal taxfreePrice = null; // 不含税单价
        BigDecimal tax = null; // 税额
        BigDecimal taxRate = billDetail.getTaxrate(); // 税率
        BigDecimal num = billDetail.getNum(); // 商品数量
        BigDecimal taxAmt = null; // 含税金额
        BigDecimal taxFreeAmt = null; // 不含税金额

        if (hsbz != null && hsbz == 1) {
            //通过含税单价计算税额、含税金额、不含税金额
            taxPrice = billDetail.getPrice(); // 含税单价
            taxAmt = taxPrice.multiply(num).setScale(2, BigDecimal.ROUND_HALF_UP);
            BigDecimal taxRateS = new BigDecimal("1").add(taxRate);
            tax = num.multiply(taxPrice).multiply(taxRate).divide(taxRateS, 2, BigDecimal.ROUND_HALF_UP);
            taxFreeAmt = taxAmt.subtract(tax);
            // 诺诺的文档公式如下，但是实际上开出来的票额和这个不一致： 在确认含税单价或者含税金额的情况下，含税标志为 1 含税，税额=round((数量*含税单价)*税率/(1+税率)),2)，不含税金额=含税金额-税额。
            // 需要先将含税单价转换为不含税单价，不含税单价进行计算，得到的结果才与票面一直
            // taxAmt = taxPrice.multiply(num);
            // tax = ((taxAmt.multiply(taxRate)).divide((new BigDecimal("1")).multiply(taxRate))).setScale(2, BigDecimal.ROUND_HALF_UP);
            // taxFreeAmt = taxPrice.subtract(tax);
        } else {
            //通过不含税单价计算税额、含税金额、不含税金额
            taxfreePrice = (billDetail.getPrice()).setScale(8, BigDecimal.ROUND_DOWN); // 不含税单价
            taxFreeAmt = taxfreePrice.multiply(num).setScale(2, BigDecimal.ROUND_HALF_UP);
            tax = num.multiply(taxfreePrice).multiply(taxRate).setScale(2, BigDecimal.ROUND_HALF_UP);
            taxAmt = taxFreeAmt.add(tax);
            // taxPrice = taxfreePrice.multiply((new BigDecimal("1")).add(taxRate)).setScale(9, BigDecimal.ROUND_HALF_UP); // 计算含税单价
            // 在确认不含税单价或者不含税金额的情况下，含税标志为 0 不含税，税额=round((数量*不含 税单价)*税率),2)，含税金额=不含税金额+税额。
            // tax = (taxfreePrice.multiply(num).multiply(taxRate)).setScale(2, BigDecimal.ROUND_HALF_UP); // 计算税额
            // taxAmt = taxFreeAmt.add(tax); // 计算含税金额
        }

        billDetail.setTaxamt(taxAmt);
        billDetail.setTaxfreeamt(taxFreeAmt);
        billDetail.setTax(tax);
        return billDetail;
    }

    /**
     * 计算单据明细规则
     */
    public static AjaxResult detailFormat(BillDetail billDetailRaw) {
        BillDetail billDetail = JSONObject.parseObject(JSONObject.toJSONString(billDetailRaw), BillDetail.class);
        Integer fphxz = billDetail.getFphxz();
        // TODO: 2023/11/16 这里被折扣行开票有问题，开票不传taxamt的话
        // 如果是被折扣行，直接返回
        if (fphxz != null && fphxz == 1) {
            return AjaxResult.error(300, "明细行为被折扣行");
        }
        //如果税率为空，直接返回
        if (billDetail.getTaxrate() == null) {
            return AjaxResult.error("税率为空", billDetail);
        }
        //如果单价或数量为空，直接返回
        /*if(billDetail.getPrice() == null && billDetail.getNum() == null){
            return AjaxResult.error("单价和数量均为空",billDetail);
        }*/
        // 如果存在扣除额，直接返回
        BigDecimal kce = billDetail.getKce();
        // 如果当前行存在扣除额
        if (kce != null && !"".equals(kce) && !"0".equals(kce)) {
            return AjaxResult.error(300, "明细行存在扣除额");
        }

        //如果不含税金额、税额、含税金额存在则校验是否相等
        if (billDetail.getTaxamt() != null && billDetail.getTaxfreeamt() != null && billDetail.getTax() != null) {
            BigDecimal dqce = billDetail.getTaxamt().subtract(billDetail.getTaxfreeamt()).subtract(billDetail.getTax());//当前差额
            if (dqce.compareTo(new BigDecimal("0")) != 0) {
                return AjaxResult.error("明细行含税金额与不含税金额、税额的和不相等", billDetail);
            }
        }

        BigDecimal ce6 = new BigDecimal("0.06");//差额6分钱
        //BigDecimal ce1 = new BigDecimal("0.01");//差额6分钱
        BigDecimal ce0 = new BigDecimal("0.00");//差额0分钱
        BigDecimal taxRate = billDetail.getTaxrate(); // 税率
        BigDecimal taxRateS = new BigDecimal("1").add(taxRate);
        Integer hsbz = null;
        BigDecimal price = null; // 单价
        BigDecimal tax = null; // 税额
        BigDecimal taxs = null;//计算出的税额
        BigDecimal num = null; // 商品数量
        BigDecimal taxAmt = null; // 含税金额
        BigDecimal taxFreeAmt = null; // 不含税金额
        /**单价、数量、含税金额、不含税金额、税额存在 校验值是否大于6分钱 */
        if (billDetail.getPrice() != null && billDetail.getNum() != null && billDetail.getTaxamt() != null
                && billDetail.getTaxfreeamt() != null && billDetail.getTax() != null) {
            if (billDetail.getHsbz() == null) {
                return AjaxResult.error("单价、数量都存在时，含税标志不能为空", billDetail);
            }

            hsbz = billDetail.getHsbz();
            BigDecimal amt = billDetail.getPrice().multiply(billDetail.getNum());
            if (hsbz == 0) {
                taxs = billDetail.getTaxfreeamt().multiply(taxRate).setScale(2, BigDecimal.ROUND_HALF_UP);
                tax = billDetail.getTax();
                BigDecimal taxce = tax.subtract(taxs);
                taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                if (taxce.compareTo(ce6) > 0) {
                    return AjaxResult.error("明细行不含税金额计算出的税额与传入税额差额大于6分钱", billDetail);
                }
                //不含税金额校验
                BigDecimal dqce = billDetail.getTaxfreeamt().subtract(amt);
                dqce = dqce.compareTo(new BigDecimal("0")) > 0 ? dqce : dqce.multiply(new BigDecimal("-1"));
                //  dqce 大于 ce6时 进入
                if (dqce.compareTo(ce6) > 0) {
                    return AjaxResult.error("明细行不含税单价、数量积与不含税金额差额大于6分钱", billDetail);
                }
            }
            if (hsbz == 1) {
                num = billDetail.getNum();
                price = billDetail.getPrice();
                taxs = billDetail.getTaxamt().multiply(taxRate).divide(taxRateS, 2, BigDecimal.ROUND_HALF_UP);
                //taxs = num.multiply(price).multiply(taxRate).divide(taxRateS).setScale(2, BigDecimal.ROUND_HALF_UP);
                tax = billDetail.getTax();
                BigDecimal taxce = tax.subtract(taxs);
                taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                if (taxce.compareTo(ce6) > 0) {
                    return AjaxResult.error("明细行含税金额计算出的税额与传入税额差额大于6分钱", billDetail);
                }
            }
            return AjaxResult.success(billDetail);
        }
        /**数量存在单价不存在 计算单价及其他金额（判断含税金额存在、或不含税金额存在，优先含税）*/
        else if (billDetail.getNum() != null && billDetail.getPrice() == null) {
            //含税金额、税额存在
            if (billDetail.getTaxamt() != null && billDetail.getTax() != null) {
                price = billDetail.getTaxamt().divide(billDetail.getNum(), 8, BigDecimal.ROUND_HALF_UP);
                taxs = billDetail.getTaxamt().multiply(taxRate).divide(taxRateS, 2, BigDecimal.ROUND_HALF_UP);
                tax = billDetail.getTax();
                BigDecimal taxce = tax.subtract(taxs);
                taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                if (taxce.compareTo(ce6) > 0) {
                    return AjaxResult.error("明细行含税金额计算出的税额与传入税额差额大于6分钱", billDetail);
                }
                taxFreeAmt = billDetail.getTaxamt().subtract(billDetail.getTax());
                billDetail.setPrice(price);
                billDetail.setHsbz(1);
                billDetail.setTaxfreeamt(taxFreeAmt);
            }
            //不含税金额、税额存在
            else if (billDetail.getTaxfreeamt() != null && billDetail.getTax() != null) {
                price = billDetail.getTaxfreeamt().divide(billDetail.getNum(), 8, BigDecimal.ROUND_HALF_UP);
                taxs = billDetail.getTaxfreeamt().multiply(taxRate).setScale(2, BigDecimal.ROUND_HALF_UP);
                tax = billDetail.getTax();
                BigDecimal taxce = tax.subtract(taxs);
                taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                if (taxce.compareTo(ce6) > 0) {
                    return AjaxResult.error("明细行不含税金额计算出的税额与传入税额差额大于6分钱", billDetail);
                }
                taxAmt = billDetail.getTaxfreeamt().add(billDetail.getTax());
                billDetail.setPrice(price);
                billDetail.setHsbz(0);
                billDetail.setTaxamt(taxAmt);
            }
            //含税金额、不含税金额存在
            else if (billDetail.getTaxamt() != null && billDetail.getTaxfreeamt() != null) {
                price = billDetail.getTaxamt().divide(billDetail.getNum(), 8, BigDecimal.ROUND_HALF_UP);
                taxs = billDetail.getTaxamt().multiply(taxRate).divide(taxRateS, 2, BigDecimal.ROUND_HALF_UP);
                tax = billDetail.getTaxamt().subtract(billDetail.getTaxfreeamt());
                BigDecimal taxce = tax.subtract(taxs);
                taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                if (taxce.compareTo(ce6) > 0) {
                    return AjaxResult.error("明细行含税金额使用公式计算出的税额与传入含税金额减去不含税金额计算出的税额差额大于6分钱", billDetail);
                }
                billDetail.setPrice(price);
                billDetail.setHsbz(1);
                billDetail.setTax(tax);
            } else {
                return AjaxResult.error("没有匹配的计算规则", billDetail);
            }
            return AjaxResult.success(billDetail);
        }
        /**单价存在数量不存在 计算数量及其他金额*/
        else if (billDetail.getPrice() != null && billDetail.getNum() == null) {
            if (billDetail.getHsbz() == null) {
                return AjaxResult.error("单价存在数量不存在时，含税标志不能为空", billDetail);
            }
            //含税标志为含税：含税金额、税额存在
            if (billDetail.getHsbz() == 1 && billDetail.getTaxamt() != null && billDetail.getTax() != null) {
                num = billDetail.getTaxamt().divide(billDetail.getPrice(), 8, BigDecimal.ROUND_HALF_UP);
                taxs = billDetail.getTaxamt().multiply(taxRate).divide(taxRateS, 2, BigDecimal.ROUND_HALF_UP);
                tax = billDetail.getTax();
                BigDecimal taxce = tax.subtract(taxs);
                taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                if (taxce.compareTo(ce6) > 0) {
                    return AjaxResult.error("明细行含税金额计算出的税额与传入税额差额大于6分钱", billDetail);
                }
                taxFreeAmt = billDetail.getTaxamt().subtract(billDetail.getTax());
                billDetail.setNum(num);
                billDetail.setTaxfreeamt(taxFreeAmt);
            }
            //含税标志为含税：不含税金额、税额存在
            else if (billDetail.getHsbz() == 1 && billDetail.getTaxfreeamt() != null && billDetail.getTax() != null) {
                taxs = billDetail.getTaxfreeamt().multiply(taxRate).setScale(2, BigDecimal.ROUND_HALF_UP);
                tax = billDetail.getTax();
                BigDecimal taxce = tax.subtract(taxs);
                taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                if (taxce.compareTo(ce6) > 0) {
                    return AjaxResult.error("明细行不含税金额计算出的税额与传入税额差额大于6分钱", billDetail);
                }
                taxAmt = billDetail.getTaxfreeamt().add(billDetail.getTax());
                num = taxAmt.divide(billDetail.getPrice(), 8, BigDecimal.ROUND_HALF_UP);
                billDetail.setNum(num);
                billDetail.setTaxamt(taxAmt);
            }
            //含税标志为不含税：含税金额、税额存在
            else if (billDetail.getHsbz() == 0 && billDetail.getTaxamt() != null && billDetail.getTax() != null) {
                taxs = billDetail.getTaxamt().multiply(taxRate).divide(taxRateS, 2, BigDecimal.ROUND_HALF_UP);
                tax = billDetail.getTax();
                BigDecimal taxce = tax.subtract(taxs);
                taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                if (taxce.compareTo(ce6) > 0) {
                    return AjaxResult.error("明细行含税金额计算出的税额与传入税额差额大于6分钱", billDetail);
                }
                taxFreeAmt = billDetail.getTaxamt().subtract(billDetail.getTax());
                num = taxFreeAmt.divide(billDetail.getPrice(), 8, BigDecimal.ROUND_HALF_UP);
                billDetail.setNum(num);
                billDetail.setTaxfreeamt(taxFreeAmt);
            }
            //含税标志为不含税：不含税金额、税额存在
            else if (billDetail.getHsbz() == 0 && billDetail.getTaxfreeamt() != null && billDetail.getTax() != null) {
                taxs = billDetail.getTaxfreeamt().multiply(taxRate).setScale(2, BigDecimal.ROUND_HALF_UP);
                tax = billDetail.getTax();
                BigDecimal taxce = tax.subtract(taxs);
                taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                if (taxce.compareTo(ce6) > 0) {
                    return AjaxResult.error("明细行不含税金额计算出的税额与传入税额差额大于6分钱", billDetail);
                }
                num = billDetail.getTaxfreeamt().divide(billDetail.getPrice(), 8, BigDecimal.ROUND_HALF_UP);
                taxAmt = billDetail.getTaxfreeamt().add(billDetail.getTax());
                billDetail.setNum(num);
                billDetail.setTaxamt(taxAmt);
            } else {
                return AjaxResult.error("没有匹配的计算规则", billDetail);
            }
            return AjaxResult.success(billDetail);
        }
        /**数量、单价存在 计算各类金额*/
        else if (billDetail.getPrice() != null && billDetail.getNum() != null) {
            if (billDetail.getHsbz() == null) {
                return AjaxResult.error("单价、数量都存在时，含税标志不能为空", billDetail);
            }
            hsbz = billDetail.getHsbz();
            num = billDetail.getNum();
            price = billDetail.getPrice();
            BigDecimal amt = price.multiply(num);
            //含税金额、税额、不含税金额均不存在 存在误差、差额
            if (billDetail.getTaxamt() == null && billDetail.getTaxfreeamt() == null && billDetail.getTax() == null) {
                if (billDetail.getHsbz() == 1) {
                    taxAmt = amt.setScale(2, BigDecimal.ROUND_HALF_UP);
                    tax = amt.multiply(taxRate).divide(taxRateS, 2, BigDecimal.ROUND_HALF_UP);
                    taxFreeAmt = taxAmt.subtract(tax);
                    billDetail.setTaxamt(taxAmt);
                    billDetail.setTax(tax);
                    billDetail.setTaxfreeamt(taxFreeAmt);
                    return AjaxResult.success(billDetail);
                } else if (billDetail.getHsbz() == 0) {
                    taxFreeAmt = amt.setScale(2, BigDecimal.ROUND_HALF_UP);
                    tax = taxFreeAmt.multiply(taxRate).setScale(2, BigDecimal.ROUND_HALF_UP);
                    taxAmt = taxFreeAmt.add(tax);
                    billDetail.setTaxamt(taxAmt);
                    billDetail.setTax(tax);
                    billDetail.setTaxfreeamt(taxFreeAmt);
                    return AjaxResult.success(billDetail);
                }
            }
            //含税金额存在 存在差额
            else if (billDetail.getTaxamt() != null) {
                if (hsbz == 1) {//含税标志为 1 含税，税额=round((数量*含税单价)*税率/(1+税率)),2)，不含税金额=含税金额-税额。
                    BigDecimal dqce = billDetail.getTaxamt().subtract(amt);
                    //是否相等
                    if (dqce.compareTo(ce0) != 0) {
                        return AjaxResult.error("明细行含税单价、数量积不等于含税金额", billDetail);
                    }
                    taxAmt = billDetail.getTaxamt();
                    tax = taxAmt.multiply(taxRate).divide(taxRateS, 2, BigDecimal.ROUND_HALF_UP);
                    taxFreeAmt = taxAmt.subtract(tax);
                    billDetail.setTaxamt(taxAmt);
                    billDetail.setTax(tax);
                    billDetail.setTaxfreeamt(taxFreeAmt);
                    return AjaxResult.success(billDetail);
                } else if (hsbz == 0) {//含税标志为 0 不含税，税额=round((数量*不含税单价)*税率),2)，含税金额=不含税金额+税额。
                    taxAmt = billDetail.getTaxamt();
                    taxFreeAmt = amt.setScale(2, BigDecimal.ROUND_HALF_UP);
                    tax = taxAmt.subtract(taxFreeAmt);
                    taxs = taxAmt.multiply(taxRate).divide(taxRateS, 2, BigDecimal.ROUND_HALF_UP);
                    BigDecimal taxce = tax.subtract(taxs);
                    taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                    if (taxce.compareTo(ce6) > 0) {
                        return AjaxResult.error("明细行含税金额使用公式计算出的税额与传入含税金额减去不含税金额（不含税单价、数量积）计算出的税额差额大于6分钱", billDetail);
                    }
                    billDetail.setTaxamt(taxAmt);
                    billDetail.setTax(tax);
                    billDetail.setTaxfreeamt(taxFreeAmt);
                    return AjaxResult.success(billDetail);
                }
            }
            //不含税金额存在 存在差额
            else if (billDetail.getTaxfreeamt() != null) {
                if (hsbz == 0) {//含税标志为 0 不含税，税额=round((数量*不含税单价)*税率),2)，含税金额=不含税金额+税额。
                    BigDecimal dqce = billDetail.getTaxfreeamt().subtract(amt);
                    //是否相等
                    if (dqce.compareTo(ce0) != 0) {
                        return AjaxResult.error("明细行不含税单价、数量积不等于不含税金额", billDetail);
                    }
                    dqce = dqce.compareTo(new BigDecimal("0")) > 0 ? dqce : dqce.multiply(new BigDecimal("-1"));
                    taxFreeAmt = billDetail.getTaxfreeamt();
                    tax = taxFreeAmt.multiply(taxRate).setScale(2, BigDecimal.ROUND_HALF_UP);
                    taxAmt = taxFreeAmt.add(tax);
                    billDetail.setTaxamt(taxAmt);
                    billDetail.setTax(tax);
                    billDetail.setTaxfreeamt(taxFreeAmt);
                    return AjaxResult.success(billDetail);
                } else if (hsbz == 1) {//含税标志为 1 含税，税额=round((数量*含税单价)*税率/(1+税率)),2)，不含税金额=含税金额-税额。
                    taxFreeAmt = billDetail.getTaxfreeamt();
                    taxAmt = amt.setScale(2, BigDecimal.ROUND_HALF_UP);
                    tax = taxAmt.subtract(taxFreeAmt);
                    taxs = taxFreeAmt.multiply(taxRate).setScale(2, BigDecimal.ROUND_HALF_UP);
                    BigDecimal taxce = tax.subtract(taxs);
                    taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                    if (taxce.compareTo(ce6) > 0) {
                        return AjaxResult.error("明细行含税金额使用公式计算出的税额与传入含税金额（含税单价、数量积）减去不含税金额计算出的税额差额大于6分钱", billDetail);
                    }
                    billDetail.setTaxamt(taxAmt);
                    billDetail.setTax(tax);
                    billDetail.setTaxfreeamt(taxFreeAmt);
                    return AjaxResult.success(billDetail);
                }
            }
            //税额存在 存在差额
            else if (billDetail.getTax() != null) {
                tax = billDetail.getTax();
                if (hsbz == 1) {//含税标志为 1 含税，税额=round((数量*含税单价)*税率/(1+税率)),2)，不含税金额=含税金额-税额。
                    taxAmt = amt.setScale(2, BigDecimal.ROUND_HALF_UP);
                    taxs = taxAmt.multiply(taxRate).divide(taxRateS, 2, BigDecimal.ROUND_HALF_UP);
                    BigDecimal taxce = tax.subtract(taxs);
                    taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                    if (taxce.compareTo(ce6) > 0) {
                        return AjaxResult.error("明细行含税单价、数量使用公式计算出的税额与传入税额差额大于6分钱", billDetail);
                    }
                    taxFreeAmt = taxAmt.subtract(tax);
                    billDetail.setTaxamt(taxAmt);
                    billDetail.setTax(tax);
                    billDetail.setTaxfreeamt(taxFreeAmt);
                    return AjaxResult.success(billDetail);
                } else if (hsbz == 0) {//含税标志为 0 不含税，税额=round((数量*不含税单价)*税率),2)，含税金额=不含税金额+税额。
                    taxFreeAmt = amt.setScale(2, BigDecimal.ROUND_HALF_UP);
                    taxs = taxFreeAmt.multiply(taxRate).setScale(2, BigDecimal.ROUND_HALF_UP);
                    BigDecimal taxce = tax.subtract(taxs);
                    taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                    if (taxce.compareTo(ce6) > 0) {
                        return AjaxResult.error("明细行不含税单价、数量使用公式计算出的税额与传入税额差额大于6分钱", billDetail);
                    }
                    taxAmt = taxFreeAmt.add(tax);
                    billDetail.setTaxamt(taxAmt);
                    billDetail.setTax(tax);
                    billDetail.setTaxfreeamt(taxFreeAmt);
                    return AjaxResult.success(billDetail);
                }
            }
        }
        /**数量、单价不存在 计算各类金额*/
        else if (billDetail.getNum() == null && billDetail.getPrice() == null) {
            //含税金额、税额存在
            if (billDetail.getTaxamt() != null && billDetail.getTax() != null) {
                taxs = billDetail.getTaxamt().multiply(taxRate).divide(taxRateS, 2, BigDecimal.ROUND_HALF_UP);
                tax = billDetail.getTax();
                BigDecimal taxce = tax.subtract(taxs);
                taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                if (taxce.compareTo(ce6) > 0) {
                    return AjaxResult.error("明细行含税金额计算出的税额与传入税额差额大于6分钱", billDetail);
                }
                taxFreeAmt = billDetail.getTaxamt().subtract(billDetail.getTax());
                billDetail.setTaxfreeamt(taxFreeAmt);
            }
            //含税金额、不含税金额存在
            else if (billDetail.getTaxamt() != null && billDetail.getTaxfreeamt() != null) {
                taxs = billDetail.getTaxamt().multiply(taxRate).divide(taxRateS, 2, BigDecimal.ROUND_HALF_UP);
                tax = billDetail.getTaxamt().subtract(billDetail.getTaxfreeamt());
                BigDecimal taxce = tax.subtract(taxs);
                taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                if (taxce.compareTo(ce6) > 0) {
                    return AjaxResult.error("明细行含税金额计算出的税额与传入税额差额大于6分钱", billDetail);
                }
                billDetail.setTax(tax);
            }
            //不含税金额、税额存在
            else if (billDetail.getTaxfreeamt() != null && billDetail.getTax() != null) {
                taxs = billDetail.getTaxfreeamt().multiply(taxRate).setScale(2, BigDecimal.ROUND_HALF_UP);
                tax = billDetail.getTax();
                BigDecimal taxce = tax.subtract(taxs);
                taxce = taxce.compareTo(new BigDecimal("0")) > 0 ? taxce : taxce.multiply(new BigDecimal("-1"));
                if (taxce.compareTo(ce6) > 0) {
                    return AjaxResult.error("明细行不含税金额计算出的税额与传入税额差额大于6分钱", billDetail);
                }
                taxAmt = billDetail.getTaxfreeamt().add(billDetail.getTax());
                billDetail.setTaxamt(taxAmt);
            } else {
                return AjaxResult.error("没有匹配的计算规则", billDetail);
            }
            return AjaxResult.success(billDetail);
        }
        return AjaxResult.error("没有匹配的计算规则", billDetail);
    }
}
